#include "stdafx.h"
#include "includes.h"

Terrain::Terrain(Shader* shader)
{
	terrainShader = shader;
	
	// Tablica pozycji wierzcholkow:
	float vertices[] = {
		-50.0,  0.0, -50.0,
		-50.0,  0.0,  50.0,
		 50.0,  0.0, -50.0,
		 50.0,  0.0,  50.0
	};

	// Wygenerowanie id i aktywacja nowego VAO:
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	// Wygenerowanie id i aktywacja nowego VBO:
	glGenBuffers(1, &vbo);
	glBindBuffer(GL_ARRAY_BUFFER, vbo);

	// Wypelnienie VBO zawartoscia:
	glBufferData(GL_ARRAY_BUFFER, 4 * 3 * sizeof(float), vertices, GL_STATIC_DRAW);
	// Od tego momentu nie trzeba juz trzymac wierzcholkow w pamieci glownej, znajduja sie juz w pamieci karty graficznej

	// Wlaczenie atrybutu wierzcholka (pozycja):
	glEnableVertexAttribArray(terrainShader->AttribLoc_vPosition);

	// Wskazanie gdzie w VBO znajduja sie wartosci atrybutu wierzcholka (pozycja):
	glVertexAttribPointer(terrainShader->AttribLoc_vPosition, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), 0);

	// Wylaczenie VAO (nie jest niezbedne):
	glBindVertexArray(0);
}

Terrain::~Terrain()
{
}

void Terrain::Render()
{
	// Wybor programu cieniujacego:
	glUseProgram(terrainShader->ProgramId);

	// Aktywacja VAO:
	glBindVertexArray(vao);

	// Ustawienie aktualnych wartosci zmiennych uniform (macierze modelu-widoku oraz projekcji):
	glUniformMatrix4fv(terrainShader->UniformLoc_modelview, 1, GL_FALSE, glm::value_ptr(Application::Instance()->viewMatrix));
	glUniformMatrix4fv(terrainShader->UniformLoc_projection, 1, GL_FALSE, glm::value_ptr(Application::Instance()->projectionMatrix));
	
	// Zadanie renderowania wierzcholkow z aktualnie aktywnego VBO (wskazanego przez aktywne VAO), w kolejnosci takiej w jakiej znajduja sie w VBO:
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	// Sprzatanie (nie jest niezbedne):
	glBindVertexArray(0);
	glUseProgram(0);
}
